@@ -6,6 +6,8 @@
 
 #include "AndroidUtils.h"
 
+#include <QFileInfo>
+
 #ifdef Q_OS_ANDROID
 
 #include <QBuffer>
@@ -13,24 +15,19 @@
 #include <QHash>
 #include <QImage>
 #include <QVariant>
-#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
-#include <QAndroidIntent>
-#include <QAndroidJniObject>
-#include <QAndroidJniExceptionCleaner>
-#include <QtAndroid>
-#else
 #include <QtCore/private/qandroidextras_p.h>
 #include <QJniObject>
-#endif
 
 #else
 
-#include <QFileInfo>
 #include <QStandardPaths>
 
 #endif
 
 using namespace std;
+#ifdef Q_OS_ANDROID
+using QNativeInterface::QAndroidApplication;
+#endif
 
 //-----------------------------------------------------------------------------
 
@@ -38,11 +35,6 @@ namespace {
 
 #ifdef Q_OS_ANDROID
 
-#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
-using QJniEnvironment = QAndroidJniEnvironment;
-using QJniObject = QAndroidJniObject;
-#endif
-
 struct AutoClose
 {
     const QJniObject& m_obj;
@@ -58,11 +50,7 @@ void takePersistableUriPermission(const QJniObject& intent,
 
 QJniObject getContext()
 {
-#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
-    return QtAndroid::androidActivity();
-#else
-    return {QNativeInterface::QAndroidApplication::context()};
-#endif
+    return {QAndroidApplication::context()};
 }
 
 QJniObject getContentResolver()
@@ -134,17 +122,6 @@ void setExtraInitialUri(QJniObject& intent, const QString& uri)
                 value.object<jstring>());
 }
 
-void startActivity(
-        const QJniObject& intent, int code,
-        const function<void (int, int, const QJniObject &)>& callback)
-{
-#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
-    QtAndroid::startActivity(intent, code, callback);
-#else
-    QtAndroidPrivate::startActivity(intent, code, callback);
-#endif
-}
-
 void startDocumentActivity(
         const char* actionField, const QString& type,
         const QString& extraInitialUri, const QString& extraTitle,
@@ -188,9 +165,10 @@ void startDocumentActivity(
     }
     if (! extraInitialUri.isEmpty() && ! QUrl(extraInitialUri).isRelative())
         setExtraInitialUri(intent, extraInitialUri);
-    startActivity(intent, 0,
-                  [takePersistablePermission, callback](int, int result,
-                  const QJniObject& data) {
+    QtAndroidPrivate::startActivity(
+                intent, 0,
+                [takePersistablePermission, callback](int, int result,
+                const QJniObject& data) {
         auto ok = QJniObject::getStaticField<jint>(
                     "android/app/Activity", "RESULT_OK");
         if (result != ok)
@@ -255,6 +233,12 @@ bool AndroidUtils::checkException()
 bool AndroidUtils::checkExists(const QString& file)
 {
 #ifdef Q_OS_ANDROID
+    if (QUrl(file).isRelative())
+        return QFileInfo::exists(file);
+    // Note: using ContentResolver::query() on persisted URIs without calling
+    // ACTION_OPEN_DOCUMENT first only works on some devices. Maybe try
+    // DocumentFile.exist(DocumentFile.fromSingleUri()) once we require a
+    // Qt version that supports androidx (see also QTBUG-73904).
     auto contentResolver = getContentResolver();
     if (! contentResolver.isValid())
         return false;
@@ -296,20 +280,6 @@ void AndroidUtils::exit()
 #endif
 }
 
-QUrl AndroidUtils::getDefaultFolder()
-{
-#ifdef Q_OS_ANDROID
-    // We don't need to know a default folder on Android anymore since we
-    // now use the Storage Access Framework.
-    return {};
-#else
-    // We could also get the home directory Qt.labs.platform.StandardPaths but
-    // we don't want a dependency on Qt.labs.platform for only one function.
-    return QUrl::fromLocalFile(
-                QStandardPaths::writableLocation(QStandardPaths::HomeLocation));
-#endif
-}
-
 QString AndroidUtils::getDisplayName([[maybe_unused]]const QString& uri)
 {
 #ifdef Q_OS_ANDROID
@@ -368,6 +338,57 @@ QStringList AndroidUtils::getPersistedUriPermissions()
     return result;
 }
 
+void AndroidUtils::initTheme([[maybe_unused]]QColor colorBackground)
+{
+#ifdef Q_OS_ANDROID
+    QAndroidApplication::runOnAndroidMainThread([=]() {
+        auto window = getContext().callObjectMethod("getWindow",
+                                                    "()Landroid/view/Window;");
+        if (! window.isValid())
+            return;
+        auto flagDrawsSystemBarBackgrounds =
+                QJniObject::getStaticField<jint>(
+                    "android/view/WindowManager$LayoutParams",
+                    "FLAG_DRAWS_SYSTEM_BAR_BACKGROUNDS");
+        window.callMethod<void>("addFlags", "(I)V",
+                                flagDrawsSystemBarBackgrounds);
+        auto flagTranslucentStatus =
+                QJniObject::getStaticField<jint>(
+                    "android/view/WindowManager$LayoutParams",
+                    "FLAG_TRANSLUCENT_STATUS");
+        window.callMethod<void>("clearFlags", "(I)V", flagTranslucentStatus);
+        auto view = window.callObjectMethod("getDecorView",
+                                            "()Landroid/view/View;");
+        // Note: getSystemUiVisibility() has been deprecated in API 30,
+        // in the future use WindowInsetsController for API >= 30
+        bool isLight = (colorBackground.lightness() > 128);
+        int visibility = view.callMethod<int>("getSystemUiVisibility", "()I");
+        auto systemUiFlagLightStatusBar =
+                QJniObject::getStaticField<jint>(
+                    "android/view/View", "SYSTEM_UI_FLAG_LIGHT_STATUS_BAR");
+        if (isLight)
+            visibility |= systemUiFlagLightStatusBar;
+        else
+            visibility &= ~systemUiFlagLightStatusBar;
+        if (QAndroidApplication::sdkVersion() >= 26)
+        {
+            auto systemUiFlagLightNavigationBar =
+                    QJniObject::getStaticField<jint>(
+                        "android/view/View", "SYSTEM_UI_FLAG_LIGHT_NAVIGATION_BAR");
+            if (isLight)
+                visibility |= systemUiFlagLightNavigationBar;
+            else
+                visibility &= ~systemUiFlagLightNavigationBar;
+        }
+        view.callMethod<void>("setSystemUiVisibility", "(I)V", visibility);
+        window.callMethod<void>("setStatusBarColor", "(I)V",
+                                colorBackground.rgba());
+        window.callMethod<void>("setNavigationBarColor", "(I)V",
+                                colorBackground.rgba());
+    });
+#endif
+}
+
 #ifdef Q_OS_ANDROID
 bool AndroidUtils::open(
         [[maybe_unused]]const QString& uri, [[maybe_unused]]QByteArray& sgf)
