@@ -7,53 +7,36 @@
 #include "GameModel.h"
 
 #include <cerrno>
-#include <cmath>
 #include <cstring>
 #include <fstream>
 #include <QClipboard>
+#include <QDateTime>
 #include <QGuiApplication>
-#include <QDebug>
 #include <QDir>
 #include <QFileInfo>
 #include <QSettings>
-#include <QTextCodec>
-#include "AndroidUtils.h"
 #include "libboardgame_base/SgfUtil.h"
 #include "libboardgame_base/TreeReader.h"
 #include "libpentobi_base/MoveMarker.h"
-#include "libpentobi_base/NodeUtil.h"
 #include "libpentobi_base/PentobiTreeWriter.h"
 #include "libpentobi_base/TreeUtil.h"
+#ifdef Q_OS_ANDROID
+#include "AndroidUtils.h"
+#endif
 
-using namespace std;
-using libboardgame_base::back_to_main_variation;
-using libboardgame_base::beginning_of_branch;
-using libboardgame_base::find_next_comment;
-using libboardgame_base::get_last_node;
 using libboardgame_base::get_letter_coord;
-using libboardgame_base::has_comment;
-using libboardgame_base::has_earlier_variation;
-using libboardgame_base::is_main_variation;
+using libboardgame_base::to_lower;
 using libboardgame_base::ArrayList;
 using libboardgame_base::SgfError;
 using libboardgame_base::TreeReader;
-using libpentobi_base::to_string_id;
 using libpentobi_base::BoardType;
-using libpentobi_base::Color;
-using libpentobi_base::ColorMap;
-using libpentobi_base::ColorMove;
 using libpentobi_base::MovePoints;
 using libpentobi_base::PentobiTree;
 using libpentobi_base::PentobiTreeWriter;
-using libpentobi_base::Piece;
 using libpentobi_base::PieceInfo;
-using libpentobi_base::PiecePoints;
 using libpentobi_base::PieceSet;
 using libpentobi_base::Point;
 using libpentobi_base::has_setup;
-using libpentobi_base::get_move_number;
-using libpentobi_base::get_moves_left;
-using libpentobi_base::get_move_node;
 
 //-----------------------------------------------------------------------------
 
@@ -149,7 +132,6 @@ GameModel::GameModel(QObject* parent)
       m_nuColors(getBoard().get_nu_colors()),
       m_nuPlayers(getBoard().get_nu_players())
 {
-    loadRecentFiles();
     initGame(m_game.get_variant());
     createPieceModels();
     updateProperties();
@@ -186,17 +168,6 @@ PieceModel* GameModel::addEmpty(const QPoint& pos)
     return result;
 }
 
-void GameModel::addRecentFile(const QString& file)
-{
-    m_recentFiles.removeAll(file);
-    m_recentFiles.prepend(file);
-    while (m_recentFiles.length() > maxRecentFiles)
-        m_recentFiles.removeLast();
-    QSettings settings;
-    settings.setValue(QStringLiteral("recentFiles"), m_recentFiles);
-    emit recentFilesChanged();
-}
-
 void GameModel::addSetup(PieceModel* pieceModel, QPointF coord)
 {
     if (! checkSetupAllowed())
@@ -233,10 +204,10 @@ void GameModel::autoSave()
     else
         settings.setValue(QStringLiteral("autosave"), getSgf());
     settings.setValue(QStringLiteral("file"), m_file);
+    m_autosaveDate = QDateTime::currentMSecsSinceEpoch();
+    settings.setValue(QStringLiteral("autosaveDate"), m_autosaveDate);
     settings.setValue(QStringLiteral("fileDate"), m_fileDate);
     settings.setValue(QStringLiteral("isModified"), m_isModified);
-    m_autosaveDate = QDateTime::currentDateTime();
-    settings.setValue(QStringLiteral("autosaveDate"), m_autosaveDate);
     QVariantList location;
     uint depth = 0;
     auto node = &m_game.get_current();
@@ -261,10 +232,7 @@ void GameModel::changeGameVariant(const QString& gameVariant)
 {
     Variant variant;
     if (! parse_variant_id(gameVariant.toLocal8Bit().constData(), variant))
-    {
-        qWarning("GameModel: invalid game variant");
         return;
-    }
     initGameVariant(variant);
     setIsModified(false);
     clearFile();
@@ -272,13 +240,16 @@ void GameModel::changeGameVariant(const QString& gameVariant)
 
 bool GameModel::checkAutosaveModifiedOutside()
 {
+#ifdef Q_OS_ANDROID
+    return false;
+#else
     QSettings settings;
-    auto autosaveDate =
-            settings.value(QStringLiteral("autosaveDate")).toDateTime();
-    return m_autosaveDate.isValid() && autosaveDate.isValid()
+    auto autosaveDate = settings.value(QStringLiteral("autosaveDate"));
+    return m_autosaveDate != 0 && autosaveDate != 0
             && m_autosaveDate != autosaveDate
             && settings.value(QStringLiteral("isModified")).toBool()
             && settings.value(QStringLiteral("autosave")).toByteArray() != getSgf();
+#endif
 }
 
 bool GameModel::checkFileExists(const QString& file)
@@ -288,12 +259,16 @@ bool GameModel::checkFileExists(const QString& file)
 
 bool GameModel::checkFileModifiedOutside()
 {
-    if (m_file.isEmpty() || ! m_fileDate.isValid())
+#ifdef Q_OS_ANDROID
+    return false;
+#else
+    if (m_file.isEmpty() || m_fileDate == 0)
         return false;
     QFileInfo fileInfo(m_file);
     if (! fileInfo.exists())
         return false;
-    return fileInfo.lastModified() != m_fileDate;
+    return fileInfo.lastModified().toMSecsSinceEpoch() != m_fileDate;
+#endif
 }
 
 /** Check if setup is allowed in the current position.
@@ -314,14 +289,6 @@ void GameModel::clearFile()
     emit fileChanged();
 }
 
-void GameModel::clearRecentFiles()
-{
-    m_recentFiles.clear();
-    QSettings settings;
-    settings.setValue(QStringLiteral("recentFiles"), m_recentFiles);
-    emit recentFilesChanged();
-}
-
 bool GameModel::createFolder(const QUrl& folder)
 {
     auto localFolder = folder.toLocalFile();
@@ -329,7 +296,6 @@ bool GameModel::createFolder(const QUrl& folder)
         m_error = QString::fromLocal8Bit(strerror(errno));
         return false;
     }
-    AndroidUtils::scanFile(localFolder);
     return true;
 }
 
@@ -366,9 +332,9 @@ void GameModel::createPieceModels(Color c)
     }
 }
 
-QString GameModel::decode(const string& s) const
+QString GameModel::decode(const string& s)
 {
-    return m_textCodec->toUnicode(s.c_str());
+    return QString::fromStdString(s);
 }
 
 void GameModel::deleteAllVar()
@@ -380,9 +346,9 @@ void GameModel::deleteAllVar()
 }
 
 
-QByteArray GameModel::encode(const QString& s) const
+QByteArray GameModel::encode(const QString& s)
 {
-    return m_textCodec->fromUnicode(s);
+    return s.toUtf8();
 }
 
 GameMove* GameModel::findMoveNext()
@@ -414,16 +380,10 @@ bool GameModel::findMove(const PieceModel& pieceModel, const QString& state,
     auto piece = pieceModel.getPiece();
     auto& bd = getBoard();
     if (piece.to_int() >= bd.get_nu_uniq_pieces())
-    {
-        qWarning("GameModel::findMove: pieceModel invalid in game variant");
         return false;
-    }
     auto transform = pieceModel.getTransform(state);
     if (! transform)
-    {
-        qWarning("GameModel::findMove: transform not found");
         return false;
-    }
     auto& info = bd.get_piece_info(piece);
     PiecePoints piecePoints = info.get_points();
     transform->transform(piecePoints.begin(), piecePoints.end());
@@ -725,14 +685,14 @@ QString GameModel::getResultMessage()
     return tr("Game ends in a tie between all players.");
 }
 
-QByteArray GameModel::getSgf() const
+QByteArray GameModel::getSgf(int indent) const
 {
     auto& tree = m_game.get_tree();
     ostringstream s;
     PentobiTreeWriter writer(s, tree);
-    writer.set_indent(-1);
+    writer.set_indent(indent);
     writer.write();
-    return QByteArray(s.str().c_str());
+    return {s.str().c_str()};
 }
 
 QString GameModel::getVariationInfo() const
@@ -848,7 +808,6 @@ void GameModel::initGame(Variant variant)
 #endif
     m_game.set_date_today();
     m_game.set_charset("UTF-8");
-    m_textCodec = QTextCodec::codecForName("UTF-8");
     updateGameInfo();
 }
 
@@ -904,53 +863,40 @@ void GameModel::keepOnlySubtree()
 
 bool GameModel::loadAutoSave()
 {
-    QSettings settings;
-    auto file = settings.value(QStringLiteral("file")).toString();
-    auto isModified = settings.value(QStringLiteral("isModified")).toBool();
-    if (! file.isEmpty() && ! isModified)
     {
-        if (! checkFileExists(file) || ! openFile(file))
-            return false;
-        updateFileInfo(file);
-        m_autosaveDate = m_fileDate;
-        settings.setValue(QStringLiteral("autosaveDate"), m_autosaveDate);
+        QSettings settings;
+        auto file = settings.value(QStringLiteral("file")).toString();
+        auto isModified =
+                settings.value(QStringLiteral("isModified")).toBool();
+        if (! file.isEmpty() && ! isModified)
+        {
+            if (! openFile(file))
+                return false;
+            updateFileInfo(file);
+            m_autosaveDate = m_fileDate;
+            settings.setValue(QStringLiteral("autosaveDate"), m_autosaveDate);
+        }
+        else
+        {
+            if (! openByteArray(settings.value(
+                                    QStringLiteral("autosave")).toByteArray()))
+                return false;
+            m_fileDate = settings.value(
+                        QStringLiteral("fileDate")).toLongLong();
+            m_autosaveDate = settings.value(
+                        QStringLiteral("autosaveDate")).toLongLong();
+            setFile(file);
+        }
+        // Sanitize isModified if value from settings is inconsistent
+        if (file.isEmpty() && ! libboardgame_base::is_empty(m_game.get_tree()))
+            isModified = true;
+        setIsModified(isModified);
     }
-    else
-    {
-        if (! openByteArray(
-                    settings.value(QStringLiteral("autosave")).toByteArray()))
-            return false;
-        m_fileDate = settings.value(QStringLiteral("fileDate")).toDateTime();
-        m_autosaveDate =
-                settings.value(QStringLiteral("autosaveDate")).toDateTime();
-        setFile(file);
-    }
-    // Sanitize isModified if value from settings is inconsistent
-    if (file.isEmpty() && ! libboardgame_base::is_empty(m_game.get_tree()))
-        isModified = true;
-    setIsModified(isModified);
     restoreAutoSaveLocation();
     updateProperties();
     return true;
 }
 
-void GameModel::loadRecentFiles()
-{
-    QSettings settings;
-    m_recentFiles =
-            settings.value(QStringLiteral("recentFiles")).toStringList();
-    QMutableListIterator i(m_recentFiles);
-    while (i.hasNext())
-    {
-        auto file = i.next();
-        if (file.isEmpty() || ! QFileInfo::exists(file))
-            i.remove();
-    }
-    while (m_recentFiles.length() > maxRecentFiles)
-        m_recentFiles.removeLast();
-    emit recentFilesChanged();
-}
-
 bool GameModel::openByteArray(const QByteArray& byteArray)
 {
     istringstream in(byteArray.constData());
@@ -1027,71 +973,6 @@ void GameModel::newGame()
     updateProperties();
 }
 
-bool GameModel::openStream(istream& in)
-{
-    bool result = true;
-    try
-    {
-        preparePositionChange();
-        TreeReader reader;
-        reader.read(in);
-        auto root = reader.get_tree_transfer_ownership();
-        m_game.init(root);
-    }
-    catch (const runtime_error& e)
-    {
-        m_error =
-                tr("Invalid Blokus SGF file. (%1)")
-                .arg(QString::fromLocal8Bit(e.what()));
-        result = false;
-    }
-    auto charSet = m_game.get_charset();
-    if (charSet.empty())
-        m_textCodec = QTextCodec::codecForName("ISO 8859-1");
-    else
-        m_textCodec = QTextCodec::codecForName(m_game.get_charset().c_str());
-    if (! m_textCodec)
-    {
-        m_textCodec = QTextCodec::codecForName("ISO 8859-1");
-        m_error = tr("Unsupported character set");
-        result = false;
-    }
-    if (! result)
-        m_game.init();
-    auto variant = to_string_id(m_game.get_variant());
-    if (variant != m_gameVariant)
-        initGameVariant(m_game.get_variant());
-    setIsModified(false);
-    updateGameInfo();
-    updateProperties();
-    return result;
-}
-
-bool GameModel::openFile(const QString& file)
-{
-    auto absoluteFile = QFileInfo(file).absoluteFilePath();
-    ifstream in(absoluteFile.toLocal8Bit().constData());
-    if (! in)
-    {
-        m_error = QString::fromLocal8Bit(strerror(errno));
-        return false;
-    }
-    if (openStream(in))
-    {
-        updateFileInfo(absoluteFile);
-        addRecentFile(absoluteFile);
-        auto& root = m_game.get_root();
-        // Show end of game position by default unless the root node has
-        // setup stones or comments, because then it might be a puzzle and
-        // we don't want to show the solution.
-        if (! has_setup(root) && ! has_comment(root) && root.has_children())
-            goEnd();
-        return true;
-    }
-    clearFile();
-    return false;
-}
-
 bool GameModel::openClipboard()
 {
     auto text = QGuiApplication::clipboard()->text();
@@ -1116,6 +997,87 @@ bool GameModel::openClipboard()
     return result;
 }
 
+bool GameModel::openFile(const QString& file)
+{
+#ifdef Q_OS_ANDROID
+    if (! QUrl(file).isRelative())
+    {
+        QByteArray sgf;
+        if (! AndroidUtils::open(file, sgf))
+        {
+            m_error = AndroidUtils::getError();
+            return false;
+        }
+        string s(sgf.constData(), sgf.size());
+        istringstream in(s);
+        if (! openStream(in))
+        {
+            clearFile();
+            return false;
+        }
+        updateFileInfo(file);
+    }
+    else
+#endif
+    {
+        auto canonicalFile = QFileInfo(file).absoluteFilePath();
+        ifstream in(canonicalFile.toLocal8Bit().constData());
+        if (! in)
+        {
+            m_error = QString::fromLocal8Bit(strerror(errno));
+            return false;
+        }
+        if (! openStream(in))
+        {
+            clearFile();
+            return false;
+        }
+        updateFileInfo(canonicalFile);
+    }
+    auto& root = m_game.get_root();
+    // Show end of game position by default unless the root node has
+    // setup stones or comments, because then it might be a puzzle and
+    // we don't want to show the solution.
+    if (! has_setup(root) && ! has_comment(root) && root.has_children())
+        goEnd();
+    return true;
+}
+
+bool GameModel::openStream(istream& in)
+{
+    bool result = true;
+    try
+    {
+        preparePositionChange();
+        TreeReader reader;
+        reader.read(in);
+        auto root = reader.get_tree_transfer_ownership();
+        m_game.init(root);
+    }
+    catch (const runtime_error& e)
+    {
+        m_error =
+                tr("Invalid Blokus SGF file. (%1)")
+                .arg(QString::fromLocal8Bit(e.what()));
+        result = false;
+    }
+    auto charSet = to_lower(m_game.get_charset());
+    if (! charSet.empty() && charSet != "utf-8" && charSet != "utf8")
+    {
+        m_error = tr("Unsupported character set");
+        result = false;
+    }
+    if (! result)
+        m_game.init();
+    auto variant = to_string_id(m_game.get_variant());
+    if (variant != m_gameVariant)
+        initGameVariant(m_game.get_variant());
+    setIsModified(false);
+    updateGameInfo();
+    updateProperties();
+    return result;
+}
+
 PieceModel* GameModel::pickNamedPiece(const QString& name,
                                       PieceModel* currentPickedPiece)
 {
@@ -1168,10 +1130,7 @@ void GameModel::playPiece(PieceModel* pieceModel, QPointF coord)
     Color c(static_cast<Color::IntType>(pieceModel->color()));
     Move mv;
     if (! findMove(*pieceModel, pieceModel->state(), coord, mv))
-    {
-        qWarning("GameModel::play: illegal move");
         return;
-    }
     preparePositionChange();
     preparePieceGameCoord(pieceModel, mv);
     pieceModel->setIsPlayed(true);
@@ -1220,12 +1179,12 @@ PieceModel* GameModel::preparePiece(GameMove* move)
     return nullptr;
 }
 
-void GameModel::preparePieceGameCoord(PieceModel* pieceModel, Move mv)
+void GameModel::preparePieceGameCoord(PieceModel* pieceModel, Move mv) const
 {
     pieceModel->setGameCoord(getGameCoord(getBoard(), mv));
 }
 
-void GameModel::preparePieceTransform(PieceModel* pieceModel, Move mv)
+void GameModel::preparePieceTransform(PieceModel* pieceModel, Move mv) const
 {
     auto& bd = getBoard();
     auto transform = bd.find_transform(mv);
@@ -1304,26 +1263,46 @@ void GameModel::restoreAutoSaveLocation()
 
 bool GameModel::save(const QString& file)
 {
+    auto sgf = getSgf(1);
+#ifdef Q_OS_ANDROID
+    if (! QUrl(file).isRelative())
+    {
+        if (! AndroidUtils::save(file, sgf))
+        {
+            m_error = AndroidUtils::getError();
+            return false;
+        }
+    }
+    else
+#endif
     {
         ofstream out(file.toLocal8Bit().constData());
-        PentobiTreeWriter writer(out, m_game.get_tree());
-        writer.set_indent(1);
-        writer.write();
+        out.write(sgf.constData(), sgf.size());
         if (! out)
         {
             m_error = QString::fromLocal8Bit(strerror(errno));
             return false;
         }
     }
-    AndroidUtils::scanFile(file);
     updateFileInfo(file);
     setIsModified(false);
-    addRecentFile(file);
     return true;
 }
 
 bool GameModel::saveAsciiArt(const QString& file)
 {
+#ifdef Q_OS_ANDROID
+    ostringstream out;
+    getBoard().write(out, false);
+    string s(out.str());
+    QByteArray array(&*s.begin(), s.size());
+    if (! AndroidUtils::save(file, array))
+    {
+        m_error = AndroidUtils::getError();
+        return false;
+    }
+    return true;
+#else
     ofstream out(file.toLocal8Bit().constData());
     getBoard().write(out, false);
     if (! out)
@@ -1331,8 +1310,8 @@ bool GameModel::saveAsciiArt(const QString& file)
         m_error = QString::fromLocal8Bit(strerror(errno));
         return false;
     }
-    AndroidUtils::scanFile(file);
     return true;
+#endif
 }
 
 template<typename T>
@@ -1501,10 +1480,9 @@ void GameModel::setTime(const QString& time)
 QString GameModel::suggestFileName(const QUrl& folder,
                                    const QString& fileEnding)
 {
-    QString suffix =
-            ! fileEnding.isEmpty()
-            && ! fileEnding.startsWith(QStringLiteral(".")) ?
-                QStringLiteral(".") + fileEnding : fileEnding;
+    QString suffix = QStringLiteral(".") + fileEnding;
+    if (folder.isEmpty())
+        return tr("Untitled") + suffix;
     auto localFolder = folder.toLocalFile();
     QString file = localFolder + '/' + tr("Untitled") + suffix;
     if (QFileInfo::exists(file))
@@ -1522,8 +1500,10 @@ QString GameModel::suggestFileName(const QUrl& folder,
 
 QString GameModel::suggestGameFileName(const QUrl& folder)
 {
+#ifndef Q_OS_ANDROID
     if (! m_file.isEmpty())
         return QUrl::fromLocalFile(m_file).fileName();
+#endif
     return suggestFileName(folder, QStringLiteral("blksgf"));
 }
 
@@ -1573,7 +1553,7 @@ void GameModel::undo()
 void GameModel::updateFileInfo(const QString& file)
 {
     setFile(file);
-    m_fileDate = QFileInfo(file).lastModified();
+    m_fileDate = QFileInfo(file).lastModified().toMSecsSinceEpoch();
 }
 
 void GameModel::updateGameInfo()
