@@ -7,23 +7,18 @@
 #ifndef PENTOBI_GAME_MODEL_H
 #define PENTOBI_GAME_MODEL_H
 
-#include <QDateTime>
 #include <QUrl>
 #include "PieceModel.h"
 #include "libpentobi_base/Game.h"
 
-class QTextCodec;
-
 using namespace std;
 using libboardgame_base::SgfNode;
 using libpentobi_base::ColorMap;
 using libpentobi_base::ColorMove;
-using libpentobi_base::Board;
 using libpentobi_base::Game;
 using libpentobi_base::Move;
 using libpentobi_base::MoveList;
 using libpentobi_base::MoveMarker;
-using libpentobi_base::ScoreType;
 using libpentobi_base::Variant;
 
 //-----------------------------------------------------------------------------
@@ -63,9 +58,8 @@ class GameModel
     Q_PROPERTY(QString gameVariant READ gameVariant NOTIFY gameVariantChanged)
     Q_PROPERTY(QString positionInfo READ positionInfo NOTIFY positionInfoChanged)
     Q_PROPERTY(QString positionInfoShort READ positionInfoShort NOTIFY positionInfoShortChanged)
-    Q_PROPERTY(QString comment READ comment WRITE setComment NOTIFY commentChanged)
+    Q_PROPERTY(QString comment MEMBER m_comment WRITE setComment NOTIFY commentChanged)
     Q_PROPERTY(QString file READ file NOTIFY fileChanged)
-    Q_PROPERTY(QStringList recentFiles READ recentFiles NOTIFY recentFilesChanged)
     Q_PROPERTY(unsigned nuColors READ nuColors NOTIFY nuColorsChanged)
     Q_PROPERTY(unsigned nuPlayers READ nuPlayers NOTIFY nuPlayersChanged)
     Q_PROPERTY(unsigned toPlay READ toPlay NOTIFY toPlayChanged)
@@ -86,7 +80,7 @@ class GameModel
     Q_PROPERTY(bool hasMoves3 READ hasMoves3 NOTIFY hasMoves3Changed)
     Q_PROPERTY(bool isBoardEmpty READ isBoardEmpty NOTIFY isBoardEmptyChanged)
     Q_PROPERTY(bool isGameOver READ isGameOver NOTIFY isGameOverChanged)
-    Q_PROPERTY(bool isModified READ isModified WRITE setIsModified NOTIFY isModifiedChanged)
+    Q_PROPERTY(bool isModified MEMBER m_isModified WRITE setIsModified NOTIFY isModifiedChanged)
     Q_PROPERTY(bool canUndo READ canUndo NOTIFY canUndoChanged)
     Q_PROPERTY(bool canGoBackward READ canGoBackward NOTIFY canGoBackwardChanged)
     Q_PROPERTY(bool canGoForward READ canGoForward NOTIFY canGoForwardChanged)
@@ -101,18 +95,16 @@ class GameModel
     Q_PROPERTY(QVariantList startingPoints2 READ startingPoints2 NOTIFY startingPoints2Changed)
     Q_PROPERTY(QVariantList startingPoints3 READ startingPoints3 NOTIFY startingPoints3Changed)
     Q_PROPERTY(QVariantList startingPointsAny READ startingPointsAny NOTIFY startingPointsAnyChanged)
-    Q_PROPERTY(QString playerName0 READ playerName0 WRITE setPlayerName0 NOTIFY playerName0Changed)
-    Q_PROPERTY(QString playerName1 READ playerName1 WRITE setPlayerName1 NOTIFY playerName1Changed)
-    Q_PROPERTY(QString playerName2 READ playerName2 WRITE setPlayerName2 NOTIFY playerName2Changed)
-    Q_PROPERTY(QString playerName3 READ playerName3 WRITE setPlayerName3 NOTIFY playerName3Changed)
-    Q_PROPERTY(QString date READ date WRITE setDate NOTIFY dateChanged)
-    Q_PROPERTY(QString time READ time WRITE setTime NOTIFY timeChanged)
-    Q_PROPERTY(QString event READ getEvent WRITE setEvent NOTIFY eventChanged)
-    Q_PROPERTY(QString round READ getRound WRITE setRound NOTIFY roundChanged)
+    Q_PROPERTY(QString playerName0 MEMBER m_playerName0 WRITE setPlayerName0 NOTIFY playerName0Changed)
+    Q_PROPERTY(QString playerName1 MEMBER m_playerName1 WRITE setPlayerName1 NOTIFY playerName1Changed)
+    Q_PROPERTY(QString playerName2 MEMBER m_playerName2 WRITE setPlayerName2 NOTIFY playerName2Changed)
+    Q_PROPERTY(QString playerName3 MEMBER m_playerName3 WRITE setPlayerName3 NOTIFY playerName3Changed)
+    Q_PROPERTY(QString date MEMBER m_date WRITE setDate NOTIFY dateChanged)
+    Q_PROPERTY(QString time MEMBER m_time WRITE setTime NOTIFY timeChanged)
+    Q_PROPERTY(QString event MEMBER m_event WRITE setEvent NOTIFY eventChanged)
+    Q_PROPERTY(QString round MEMBER m_round WRITE setRound NOTIFY roundChanged)
 
 public:
-    static constexpr int maxRecentFiles = 9;
-
     static Variant getInitialGameVariant();
 
 
@@ -130,8 +122,6 @@ public:
         there is no piece at this location. */
     Q_INVOKABLE PieceModel* addEmpty(const QPoint& pos);
 
-    Q_INVOKABLE void clearRecentFiles();
-
     Q_INVOKABLE bool createFolder(const QUrl& folder);
 
     Q_INVOKABLE void deleteAllVar();
@@ -222,7 +212,7 @@ public:
 
     Q_INVOKABLE QString getResultMessage();
 
-    Q_INVOKABLE bool checkFileExists(const QString& file);
+    Q_INVOKABLE static bool checkFileExists(const QString& file);
 
     Q_INVOKABLE bool checkFileModifiedOutside();
 
@@ -239,19 +229,19 @@ public:
 
     Q_INVOKABLE PieceModel* previousPiece(PieceModel* currentPickedPiece);
 
-    Q_INVOKABLE QString suggestFileName(const QUrl& folder,
-                                        const QString& fileEnding);
+    Q_INVOKABLE static QString suggestFileName(const QUrl& folder,
+                                               const QString& fileEnding);
 
     Q_INVOKABLE QString suggestGameFileName(const QUrl& folder);
 
-    Q_INVOKABLE QString suggestNewFolderName(const QUrl& folder);
+    Q_INVOKABLE static QString suggestNewFolderName(const QUrl& folder);
 
     Q_INVOKABLE QString getError() const { return m_error; }
 
     Q_INVOKABLE QVariantList getPieceModels(int color);
 
 
-    QByteArray getSgf() const;
+    QByteArray getSgf(int indent = -1) const;
 
     void setComment(const QString& comment);
 
@@ -263,8 +253,6 @@ public:
 
     const QString& file() const { return m_file; }
 
-    const QString& comment() const { return m_comment; }
-
     unsigned nuColors() const { return m_nuColors; }
 
     unsigned nuPlayers() const { return m_nuPlayers; }
@@ -305,8 +293,6 @@ public:
 
     bool isGameOver() const { return m_isGameOver; }
 
-    bool isModified() const { return m_isModified; }
-
     bool canUndo() const { return m_canUndo; }
 
     bool canGoBackward() const { return m_canGoBackward; }
@@ -323,8 +309,6 @@ public:
 
     bool isMainVar() const { return m_isMainVar; }
 
-    const QStringList& recentFiles() const { return m_recentFiles; }
-
     const QVariantList& startingPoints0() const { return m_startingPoints0; }
 
     const QVariantList& startingPoints1() const { return m_startingPoints1; }
@@ -335,26 +319,6 @@ public:
 
     const QVariantList& startingPointsAny() const { return m_startingPointsAny; }
 
-    const QString& playerName0() const { return m_playerName0; }
-
-    const QString& playerName1() const { return m_playerName1; }
-
-    const QString& playerName2() const { return m_playerName2; }
-
-    const QString& playerName3() const { return m_playerName3; }
-
-    const QString& date() const { return m_date; }
-
-    const QString& time() const { return m_time; }
-
-    // Avoid conflict with QObject::event()
-    const QString& getEvent() const { return m_event; }
-
-    // Avoid conflict with round(), which cannot be resolved by using fully
-    // qualified std::round(), because with many GCC versions it's in the
-    // global namespace (http://stackoverflow.com/questions/1882689)
-    const QString& getRound() const { return m_round; }
-
     void setIsModified(bool isModified);
 
     void setMoveAnnotationAtNode(const SgfNode& node,
@@ -469,8 +433,6 @@ signals:
 
     void nuPlayersChanged();
 
-    void recentFilesChanged();
-
     void startingPoints0Changed();
 
     void startingPoints1Changed();
@@ -530,11 +492,16 @@ private:
 
     QString m_round;
 
-    QStringList m_recentFiles;
+    // Last-modified date of current file.
+    // Uses MSecsSinceEpoch because storing QDateTime as QVariant in QSettings
+    // can trigger a bug in Qt 5.15 that sometimes causes crashes in
+    // operator<<(QDataStream&, QTimeZone const&) invoked by
+    // QSettings::~QSettings. Seems to fixed in Qt 6.
+    qint64 m_fileDate = 0;
 
-    QDateTime m_fileDate;
-
-    QDateTime m_autosaveDate;
+    // Date of last auto-save.
+    // Uses MSecsSinceEpoch (see m_fileDate).
+    qint64 m_autosaveDate = 0;
 
     unsigned m_nuColors;
 
@@ -612,8 +579,6 @@ private:
 
     QVariantList m_tmpPoints;
 
-    QTextCodec* m_textCodec;
-
     unique_ptr<MoveList> m_legalMoves;
 
     unsigned m_legalMoveIndex;
@@ -622,8 +587,6 @@ private:
     unique_ptr<MoveMarker> m_marker;
 
 
-    void addRecentFile(const QString& file);
-
     bool checkSetupAllowed() const;
 
     void clearFile();
@@ -632,9 +595,9 @@ private:
 
     void createPieceModels(Color c);
 
-    QString decode(const string& s) const;
+    static QString decode(const string& s);
 
-    QByteArray encode(const QString& s) const;
+    static QByteArray encode(const QString& s);
 
     bool findMove(const PieceModel& pieceModel, const QString& state,
                   QPointF coord, Move& mv) const;
@@ -649,15 +612,13 @@ private:
 
     void initGameVariant(Variant variant);
 
-    void loadRecentFiles();
-
     bool openStream(istream& in);
 
     void prepareFindMove();
 
-    void preparePieceGameCoord(PieceModel* pieceModel, Move mv);
+    void preparePieceGameCoord(PieceModel* pieceModel, Move mv) const;
 
-    void preparePieceTransform(PieceModel* pieceModel, Move mv);
+    void preparePieceTransform(PieceModel* pieceModel, Move mv) const;
 
     void preparePositionChange();
 
