@@ -540,8 +540,8 @@ function openFileBlocking(file, displayName) {
         gameView.showPieces()
 }
 
-function openFileUrl() {
-    openFile(getFileFromUrl(openDialog.item.fileUrl), "")
+function openFileUrl(fileUrl) {
+    openFile(getFileFromUrl(fileUrl), "")
 }
 
 function openClipboard() {
@@ -739,7 +739,8 @@ function saveAs() {
         androidUtils.openSaveDialog(file, name)
     } else {
         var dialog = saveDialog.get()
-        dialog.name = gameModel.suggestGameFileName(folder)
+        if (! useNativeFileDialog)
+            dialog.name = gameModel.suggestGameFileName(folder)
         dialog.open()
     }
 }
