@@ -139,6 +139,17 @@
         <translation type="unfinished"></translation>
     </message>
 </context>
+<context>
+    <name>AsciiArtSaveDialogNative</name>
+    <message>
+        <source>Export ASCII Art</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Text files</source>
+        <translation type="unfinished"></translation>
+    </message>
+</context>
 <context>
     <name>BoardContextMenu</name>
     <message>
@@ -523,6 +534,21 @@
         <translation type="unfinished"></translation>
     </message>
 </context>
+<context>
+    <name>ImageSaveDialogNative</name>
+    <message>
+        <source>Save Image</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>PNG image files</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>JPEG image files</source>
+        <translation type="unfinished"></translation>
+    </message>
+</context>
 <context>
     <name>InitialRatingDialog</name>
     <message>
@@ -999,10 +1025,6 @@
         <source>About Pentobi</source>
         <translation type="unfinished"></translation>
     </message>
-    <message>
-        <source>Report Bug</source>
-        <translation type="unfinished"></translation>
-    </message>
 </context>
 <context>
     <name>MenuRecentFiles</name>
@@ -1107,6 +1129,17 @@
         <translation type="unfinished"></translation>
     </message>
 </context>
+<context>
+    <name>OpenDialogNative</name>
+    <message>
+        <source>Open</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Blokus games</source>
+        <translation type="unfinished"></translation>
+    </message>
+</context>
 <context>
     <name>PentobiFileDialog</name>
     <message>
@@ -1126,19 +1159,6 @@
         <translation type="unfinished"></translation>
     </message>
 </context>
-<context>
-    <name>PentobiMenuItem</name>
-    <message>
-        <source>Ctrl</source>
-        <extracomment>Shortcut modifier key as displayed in menu item text (abbreviate if long)</extracomment>
-        <translation type="unfinished"></translation>
-    </message>
-    <message>
-        <source>Shift</source>
-        <extracomment>Shortcut modifier key as displayed in menu item text (abbreviate if long)</extracomment>
-        <translation type="unfinished"></translation>
-    </message>
-</context>
 <context>
     <name>PentobiToolBar</name>
     <message>
@@ -1337,6 +1357,17 @@
         <translation type="unfinished"></translation>
     </message>
 </context>
+<context>
+    <name>SaveDialogNative</name>
+    <message>
+        <source>Save</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Blokus games</source>
+        <translation type="unfinished"></translation>
+    </message>
+</context>
 <context>
     <name>TableModel</name>
     <message>
