@@ -137,6 +137,17 @@
         <translation>Archivos de texto</translation>
     </message>
 </context>
+<context>
+    <name>AsciiArtSaveDialogNative</name>
+    <message>
+        <source>Export ASCII Art</source>
+        <translation>Exportar arte ASCII</translation>
+    </message>
+    <message>
+        <source>Text files</source>
+        <translation>Archivos de texto</translation>
+    </message>
+</context>
 <context>
     <name>BoardContextMenu</name>
     <message>
@@ -521,6 +532,21 @@
         <translation>Archivos de imagen JPEG</translation>
     </message>
 </context>
+<context>
+    <name>ImageSaveDialogNative</name>
+    <message>
+        <source>Save Image</source>
+        <translation>Guardar imagen</translation>
+    </message>
+    <message>
+        <source>PNG image files</source>
+        <translation>Archivos de imagen PNG</translation>
+    </message>
+    <message>
+        <source>JPEG image files</source>
+        <translation>Archivos de imagen JPEG</translation>
+    </message>
+</context>
 <context>
     <name>InitialRatingDialog</name>
     <message>
@@ -997,10 +1023,6 @@
         <source>About Pentobi</source>
         <translation>Acerca de Pentobi</translation>
     </message>
-    <message>
-        <source>Report Bug</source>
-        <translation>Informe de errores</translation>
-    </message>
 </context>
 <context>
     <name>MenuRecentFiles</name>
@@ -1105,6 +1127,17 @@
         <translation>Partidas de Blokus</translation>
     </message>
 </context>
+<context>
+    <name>OpenDialogNative</name>
+    <message>
+        <source>Open</source>
+        <translation>Abrir</translation>
+    </message>
+    <message>
+        <source>Blokus games</source>
+        <translation>Partidas de Blokus</translation>
+    </message>
+</context>
 <context>
     <name>PentobiFileDialog</name>
     <message>
@@ -1335,6 +1368,17 @@
         <translation>Partidas de Blokus</translation>
     </message>
 </context>
+<context>
+    <name>SaveDialogNative</name>
+    <message>
+        <source>Save</source>
+        <translation>Guardar</translation>
+    </message>
+    <message>
+        <source>Blokus games</source>
+        <translation>Partidas de Blokus</translation>
+    </message>
+</context>
 <context>
     <name>TableModel</name>
     <message>
