@@ -1,9 +1,9 @@
-<?xml version="1.0" ?><!DOCTYPE TS><TS language="nb_NO" version="2.1">
+<?xml version="1.0" ?><!DOCTYPE TS><TS version="2.1" language="nb_NO">
 <context>
     <name>AboutDialog</name>
     <message>
         <source>Copyright © 2011–%1 Markus Enzenberger</source>
-        <translation>Copyright © 2011–%1 Markus Enzenberger</translation>
+        <translation>Opphavsrett © 2011–%1 Markus Enzenberger</translation>
     </message>
     <message>
         <source>Computer opponent for the board game Blokus</source>
@@ -192,6 +192,24 @@
         <translation>Nivå %1</translation>
     </message>
 </context>
+<context>
+    <name>DocbookReader</name>
+    <message>
+        <source>Previous</source>
+        <extracomment>Go to previous page of user manual</extracomment>
+        <translation type="unfinished"/>
+    </message>
+    <message>
+        <source>Contents</source>
+        <extracomment>Go to table of contents of user manual</extracomment>
+        <translation type="unfinished"/>
+    </message>
+    <message>
+        <source>Next</source>
+        <extracomment>Go to next page of user manual</extracomment>
+        <translation type="unfinished"/>
+    </message>
+</context>
 <context>
     <name>ExportImageDialog</name>
     <message>
@@ -199,25 +217,6 @@
         <translation>Bildebredde:</translation>
     </message>
 </context>
-<context>
-    <name>FileDialog</name>
-    <message>
-        <source>Overwrite file?</source>
-        <translation>Overskriv fil?</translation>
-    </message>
-    <message>
-        <source>Open</source>
-        <translation>Åpne</translation>
-    </message>
-    <message>
-        <source>Save</source>
-        <translation>Lagre</translation>
-    </message>
-    <message>
-        <source>All files</source>
-        <translation>Alle filer</translation>
-    </message>
-</context>
 <context>
     <name>GameInfoDialog</name>
     <message>
@@ -504,7 +503,7 @@
     <name>HelpWindow</name>
     <message>
         <source>Pentobi Help</source>
-        <translation>Pentobi-hjelp</translation>
+        <translation type="unfinished"/>
     </message>
 </context>
 <context>
@@ -568,10 +567,6 @@
         <source>Your rating stays at %1.</source>
         <translation>Din vurdering forblir %1.</translation>
     </message>
-    <message>
-        <source>No permission to access storage</source>
-        <translation>Mangler lagringstilgang</translation>
-    </message>
     <message>
         <source>Delete all rating information for the current game variant?</source>
         <translation>Slett all vurderingsinformasjon fra nåværende spillvariant?</translation>
@@ -884,11 +879,11 @@
     </message>
     <message>
         <source>Pentobi failed to generate a move.</source>
-        <translation type="unfinished"/>
+        <translation>Pentobi klarte ikke å generere et trekk.</translation>
     </message>
     <message>
         <source>Press back again to exit</source>
-        <translation type="unfinished"/>
+        <translation>Trykk «Tilbake» igjen for å avslutte.</translation>
     </message>
 </context>
 <context>
@@ -1007,19 +1002,6 @@
         <translation>Innrapporter feil</translation>
     </message>
 </context>
-<context>
-    <name>MenuItem</name>
-    <message>
-        <source>Ctrl</source>
-        <extracomment>Shortcut modifier key as displayed in menu item text (abbreviate if long)</extracomment>
-        <translation>Ctrl</translation>
-    </message>
-    <message>
-        <source>Shift</source>
-        <extracomment>Shortcut modifier key as displayed in menu item text (abbreviate if long)</extracomment>
-        <translation>Shift</translation>
-    </message>
-</context>
 <context>
     <name>MenuRecentFiles</name>
     <message>
@@ -1123,6 +1105,110 @@
         <translation>Blokus-spill</translation>
     </message>
 </context>
+<context>
+    <name>PentobiFileDialog</name>
+    <message>
+        <source>Overwrite file?</source>
+        <translation>Overskriv fil?</translation>
+    </message>
+    <message>
+        <source>Open</source>
+        <translation>Åpne</translation>
+    </message>
+    <message>
+        <source>Save</source>
+        <translation>Lagre</translation>
+    </message>
+    <message>
+        <source>All files</source>
+        <translation>Alle filer</translation>
+    </message>
+</context>
+<context>
+    <name>PentobiMenuItem</name>
+    <message>
+        <source>Ctrl</source>
+        <extracomment>Shortcut modifier key as displayed in menu item text (abbreviate if long)</extracomment>
+        <translation>Ctrl</translation>
+    </message>
+    <message>
+        <source>Shift</source>
+        <extracomment>Shortcut modifier key as displayed in menu item text (abbreviate if long)</extracomment>
+        <translation>Shift</translation>
+    </message>
+</context>
+<context>
+    <name>PentobiToolBar</name>
+    <message>
+        <source>Start a new game</source>
+        <translation>Start nytt spill</translation>
+    </message>
+    <message>
+        <source>Start a rated game</source>
+        <translation>Start vurdert spill</translation>
+    </message>
+    <message>
+        <source>Set the colors played by the computer</source>
+        <translation>Sett fargene spilt av datamaskinen</translation>
+    </message>
+    <message>
+        <source>Make the computer continue to play the current color</source>
+        <translation>Få datamaskinen til å fortsette å spille gjeldende farge</translation>
+    </message>
+    <message>
+        <source>Make the computer play the current color</source>
+        <translation>Få datamaskinen til å spille gjeldende farge</translation>
+    </message>
+    <message>
+        <source>Go to beginning of game</source>
+        <translation>Gå til begynnelsen av spillet</translation>
+    </message>
+    <message>
+        <source>Go ten moves backward</source>
+        <translation>Gå ti trekk tilbake</translation>
+    </message>
+    <message>
+        <source>Go one move backward</source>
+        <translation>Gå ett steg bakover</translation>
+    </message>
+    <message>
+        <source>Go one move forward</source>
+        <translation>Gå ett steg forover</translation>
+    </message>
+    <message>
+        <source>Go ten moves forward</source>
+        <translation>Gå ti trekk forover</translation>
+    </message>
+    <message>
+        <source>Go to end of moves</source>
+        <translation>Gå til trekkslutt</translation>
+    </message>
+    <message>
+        <source>Go to previous variation</source>
+        <translation>Gå til forrige variasjon</translation>
+    </message>
+    <message>
+        <source>Go to next variation</source>
+        <translation>Gå til neste variasjon</translation>
+    </message>
+    <message>
+        <source>Abort game analysis</source>
+        <translation>Avbryt spillanalyse</translation>
+    </message>
+    <message>
+        <source>Abort computer move</source>
+        <translation>Avbryt datamaskintrekk</translation>
+    </message>
+    <message>
+        <source>Undo move</source>
+        <extracomment>Tooltip for Undo button</extracomment>
+        <translation>Angre trekk</translation>
+    </message>
+    <message>
+        <source>Open menu</source>
+        <translation>Åpne meny</translation>
+    </message>
+</context>
 <context>
     <name>RatingDialog</name>
     <message>
@@ -1292,78 +1378,6 @@
         <translation>Uavgjort</translation>
     </message>
 </context>
-<context>
-    <name>ToolBar</name>
-    <message>
-        <source>Start a new game</source>
-        <translation>Start nytt spill</translation>
-    </message>
-    <message>
-        <source>Start a rated game</source>
-        <translation>Start vurdert spill</translation>
-    </message>
-    <message>
-        <source>Set the colors played by the computer</source>
-        <translation>Sett fargene spilt av datamaskinen</translation>
-    </message>
-    <message>
-        <source>Make the computer continue to play the current color</source>
-        <translation>Få datamaskinen til å fortsette å spille gjeldende farge</translation>
-    </message>
-    <message>
-        <source>Make the computer play the current color</source>
-        <translation>Få datamaskinen til å spille gjeldende farge</translation>
-    </message>
-    <message>
-        <source>Go to beginning of game</source>
-        <translation>Gå til begynnelsen av spillet</translation>
-    </message>
-    <message>
-        <source>Go ten moves backward</source>
-        <translation>Gå ti trekk tilbake</translation>
-    </message>
-    <message>
-        <source>Go one move backward</source>
-        <translation>Gå ett steg bakover</translation>
-    </message>
-    <message>
-        <source>Go one move forward</source>
-        <translation>Gå ett steg forover</translation>
-    </message>
-    <message>
-        <source>Go ten moves forward</source>
-        <translation>Gå ti trekk forover</translation>
-    </message>
-    <message>
-        <source>Go to end of moves</source>
-        <translation>Gå til trekkslutt</translation>
-    </message>
-    <message>
-        <source>Go to previous variation</source>
-        <translation>Gå til forrige variasjon</translation>
-    </message>
-    <message>
-        <source>Go to next variation</source>
-        <translation>Gå til neste variasjon</translation>
-    </message>
-    <message>
-        <source>Abort game analysis</source>
-        <translation>Avbryt spillanalyse</translation>
-    </message>
-    <message>
-        <source>Abort computer move</source>
-        <translation>Avbryt datamaskintrekk</translation>
-    </message>
-    <message>
-        <source>Undo move</source>
-        <extracomment>Tooltip for Undo button</extracomment>
-        <translation>Angre trekk</translation>
-    </message>
-    <message>
-        <source>Open menu</source>
-        <translation type="unfinished"/>
-    </message>
-</context>
 <context>
     <name>main</name>
     <message>
